
package eu.dnetlib.dhp.oa.graph.dump.gcat;

import java.io.Serializable;
import java.io.StringWriter;
import java.security.Key;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.apache.avro.generic.GenericData;
import org.apache.commons.lang3.StringUtils;
import org.apache.spark.util.LongAccumulator;

import eu.dnetlib.dhp.oa.graph.dump.Constants;
import eu.dnetlib.dhp.schema.dump.gcat.CatalogueEntry;
import eu.dnetlib.dhp.schema.dump.gcat.Group;
import eu.dnetlib.dhp.schema.dump.gcat.Tag;
import eu.dnetlib.dhp.schema.dump.oaf.*;
import eu.dnetlib.dhp.schema.dump.oaf.KeyValue;
import eu.dnetlib.dhp.schema.oaf.*;
import eu.dnetlib.dhp.schema.oaf.Author;
import eu.dnetlib.dhp.schema.oaf.Qualifier;
import org.omg.CORBA.UNKNOWN;

public class Mapper implements Serializable {

	private static final List<String> publishers = Arrays
		.asList("zenodo", "hal", "figshare", "digital-csic", "dans", "datacite");
	private static final List<String> access = Arrays.asList("open", "closed", "embargoed", "restricted");

	public static <I extends eu.dnetlib.dhp.schema.oaf.Result> CatalogueEntry map(I input,
		Map<String, LongAccumulator> map) {

		final CatalogueEntry out = new CatalogueEntry();
		Optional<Qualifier> ort = Optional.ofNullable(input.getResulttype());
		List<KeyValue> externals = new ArrayList<>();
		Set<String> urlSet = new HashSet<>();
		Set<String> cfSet = new HashSet<>();
		Set<String> hbSet = new HashSet<>();
		Set<String> countrySet = new HashSet<>();
		Set<String> groups = new HashSet<>();
		List<Group> groupList = new ArrayList<>();
		if (ort.isPresent()) {
			switch (ort.get().getClassid()) {
				case "publication":
					Optional<Journal> oJournal = Optional
						.ofNullable(((Publication) input).getJournal());
					if (oJournal.isPresent()) {
						Journal value = oJournal.get();
						externals
							.add(
								KeyValue
									.newInstance(
										"Risis2_Publishing:Journal",
										// "Journal",
										value.getName() + ", " + value.getVol() + ", " + value.getIss()));
					}
					out.setUrl(Constants.PUBLICATION_URL + input.getId().substring(3));
					externals.add(KeyValue.newInstance("system:type", "publication"));
					break;
				case "dataset":
					eu.dnetlib.dhp.schema.oaf.Dataset id = (eu.dnetlib.dhp.schema.oaf.Dataset) input;
					out
						.setVersion(
							Optional
								.ofNullable(id.getVersion())
								.map(v -> v.getValue())
								.orElse(""));
					out.setUrl(Constants.DATASET_URL + input.getId().substring(3));
					externals.add(KeyValue.newInstance("system:type", "dataset"));
					break;
				case "software":
					eu.dnetlib.dhp.schema.oaf.Software is = (eu.dnetlib.dhp.schema.oaf.Software) input;
					Optional
						.ofNullable(is.getCodeRepositoryUrl())
						.ifPresent(value -> urlSet.add(value.getValue()));
					Optional
						.ofNullable(is.getDocumentationUrl())
						.ifPresent(value -> value.forEach(v -> urlSet.add(v.getValue())));

					externals
						.add(
							KeyValue
								.newInstance(
									"Programming Language", Optional
										.ofNullable(is.getProgrammingLanguage())
										.map(v -> v.getClassname())
										.orElse("")));
//						.ifPresent(
//							value -> externals.add(KeyValue.newInstance("Programming Language", value.getClassname())));
					out.setUrl(Constants.SOFTWARE_URL + input.getId().substring(3));
					externals.add(KeyValue.newInstance("system:type", "software"));
					break;

				case "other":
					out.setUrl(Constants.ORP_URL + input.getId().substring(3));
					externals.add(KeyValue.newInstance("system:type", "other"));
					break;

			}

			out.setLicense_id(Constants.DEFAULT_LICENCE_ID);

			Optional<List<Author>> oauth = Optional
				.ofNullable(input.getAuthor());
			List<String> authList = new ArrayList<>();
			if (oauth.isPresent()) {
				authList = oauth.get().stream().map(v -> getAuthor(v)).collect(Collectors.toList());
			}
			if (authList.size() > 0) {
				authList.forEach(a -> externals.add(KeyValue.newInstance("Risis2_Attribution:Author", a)));
				// authList.forEach(a -> externals.add(KeyValue.newInstance("Author", a)));
			}

			String accessr = Optional
				.ofNullable(input.getBestaccessright())
				.map(
					value -> value.getClassid())
				.orElse("");
			if (access.contains(accessr.toLowerCase())) {
				groupList.add(Group.newInstance(null, accessr.toLowerCase().trim()));
			}
			if (!accessr.equals("")) {
				externals
					.add(
						KeyValue
							.newInstance(
								"AccessMode:Access Right", input.getBestaccessright().getClassname()));
			}

			Optional
				.ofNullable(input.getCollectedfrom())
				.ifPresent(
					value -> value
						.forEach(v -> cfSet.add(v.getValue())));

			Optional<List<Field<String>>> ocont = Optional
				.ofNullable(input.getContributor());
			if (ocont.isPresent()) {
				ocont
					.get()
					.forEach(
						v -> externals
							.add(
								KeyValue
									.newInstance(
										"Risis2_Attribution:Contributor",
										v.getValue())));
				// .forEach(v -> externals.add(KeyValue.newInstance("Contributor", v.getValue())));
			}

			Optional
				.ofNullable(input.getCountry())
				.ifPresent(
					value -> value
						.forEach(v -> countrySet.add(v.getClassname())));

			Optional
				.ofNullable(input.getDescription())
				.ifPresent(value ->

				getDescription(out, externals, value));

			externals
				.add(
					KeyValue
						.newInstance(
							"AccessMode:Embargo End Date", Optional
								.ofNullable(input.getEmbargoenddate())
								.map(value -> value.getValue())
								.orElse("")));

			final Set<String> formatSet = new HashSet<>();
			Optional
				.ofNullable(input.getFormat())
				.ifPresent(value -> value.forEach(f -> formatSet.add(f.getValue())));

			String id = input.getId();
			// id = id.substring(0, id.lastIndexOf(":") + 1) + "a" + id.substring(id.lastIndexOf(":") + 1);
			out.setName(id.substring(id.indexOf('|') + 1).replace(":", "-"));

			final Set<String> itSet = new HashSet<>();
			Optional
				.ofNullable(input.getInstance())
				.ifPresent(
					value -> value
						.forEach(v -> {

							Optional
								.ofNullable(v.getHostedby())
								.ifPresent(hb -> hbSet.add(hb.getValue()));

							Optional
								.ofNullable(v.getUrl())
								.ifPresent(u -> u.forEach(url -> urlSet.add(url)));

							Optional
								.ofNullable(v.getInstancetype())
								.ifPresent(it -> itSet.add(it.getClassname()));

						}));

			externals
				.add(
					KeyValue
						.newInstance(
							"Language", Optional
								.ofNullable(input.getLanguage())
								.map(value -> {
									String lang = value.getClassname();
									if(lang.toLowerCase().equals(Constants.UNKNOWN)){
										return "";
									}
									return  lang;
								})
								.orElse("")));

			List<StructuredProperty> iTitle = Optional
				.ofNullable(input.getTitle())
				.map(
					value -> value
						.stream()
						.filter(t -> t.getQualifier().getClassid().equalsIgnoreCase("main title"))
						.collect(Collectors.toList()))
				.orElse(new ArrayList<>());

			if (iTitle.size() > 0) {
				out.setTitle(textReplacement(iTitle.get(0).getValue()));
			} else {
				out.setTitle("");
			}

			Optional
				.ofNullable(input.getPid())
				.ifPresent(
					value -> value
						.forEach(
							v -> {
								if (v.getQualifier().getClassid().equalsIgnoreCase("DOI")) {
									externals
										.add(
											KeyValue
												.newInstance("Identity:PID", "https://www.doi.org/" + v.getValue()));
								} else {
									externals
										.add(
											KeyValue
												.newInstance(
													"Identity:PID",
													v.getQualifier().getClassid() + ":" + v.getValue()));
								}

							}));

			externals
				.add(
					KeyValue
						.newInstance(
							"Risis2_Publishing:Publication Date", Optional
								// "Publication Date", Optional
								.ofNullable(input.getDateofacceptance())
								.map(value -> value.getValue())
								.orElse("")));

			String publisher = Optional
				.ofNullable(input.getPublisher())
				.map(value -> value.getValue())
				.orElse("");

			if (!publisher.equals("")) {
				groups.add(publisher.toLowerCase().replace(".", "-"));
				externals
					.add(
						KeyValue
							.newInstance(
								"Risis2_Publishing:Publisher", publisher));
			}

			Set<String> tagsSet = new HashSet<>();
			Optional
				.ofNullable(input.getSubject())
				.ifPresent(
					value -> value
						.forEach(
							s -> {
								String classId = s.getQualifier().getClassid();
								String prefix = "";
								if (!(classId.equals("keyword") || classId.toLowerCase().equals(Constants.UNKNOWN)) &&
									StringUtils.isNotEmpty(classId)) {
									prefix = classId + ".";
								}
								String tag = prefix + s.getValue();
								tag = tagReplacements(tag);
								tagsSet.add(tag);
							}));

			cfSet.remove("Unknown Repository");
			externals.add(KeyValue.newInstance("Risis2_Publishing:Collected From", getListOfValues(cfSet)));

			hbSet.remove("Unknown Repository");
			externals.add(KeyValue.newInstance("Risis2_Publishing:Hosted By", getListOfValues(hbSet)));

			cfSet.forEach(cf -> groups.add(cf.toLowerCase().replace(".", "-")));
			hbSet.forEach(hb -> groups.add(hb.toLowerCase().replace(".", "-")));

			groups.forEach(g -> {
				if (publishers.contains(g.trim())) {
					groupList.add(Group.newInstance(null, g.trim()));
				}
			});
			out.setGroups(groupList);
			urlSet.stream().forEach(url -> externals.add(KeyValue.newInstance("Identity:URL", url)));
			externals.add(KeyValue.newInstance("Country", getListOfValues(countrySet)));
			externals.add(KeyValue.newInstance("Format", getListOfValues(formatSet)));
			externals.add(KeyValue.newInstance("Resource Type", getListOfValues(itSet)));
			List<Tag> tags = new ArrayList<>();
			List<String> kws = new ArrayList<>();
			tagsSet.forEach(tag -> {
				if (tag.endsWith(",") || tag.endsWith(";") || tag.endsWith(".")) {
					tag = tag.substring(0, tag.length() - 1);
				}
				if (tag.matches("^[a-zA-Z0-9_. -]*$") && tag.length() > 1 && tag.length() < 101) {
					tags.add(Tag.newInstance(tag));
				} else {
					kws.add(tag);
				}

			});

			out.setTags(tags);
			if (kws.size() > 0) {
				kws.forEach(k -> externals.add(KeyValue.newInstance("keyword", k)));
			}
			out.setExtras(externals);

		}
		if (out == null)
			map.get("void_records").add(1);

		map.get("dumped_records").add(1);
		return out;
	}

	public static String tagReplacements(String tag) {
		return tag
			.replace("&", " and ")
			// .replace(" ", "_")
			.replace("(", "_")
			.replace(")", "_")
			.replace("/", "_")
			.replace("\\", "_")
			.replace("[", "")
			.replace("]", "")
			.trim();

	}

	public static String textReplacement(String text){
		return text
				.replace("‘", "\"").replace("’", "\"")
				.replace("“", "\"").replace("”", "\"");
	}

	private static String getAuthor(Author v) {
		String author = v.getFullname();
		Optional<List<StructuredProperty>> oPid = Optional.ofNullable(v.getPid());
		if (oPid.isPresent()) {
			List<String> oList = oPid
				.get()
				.stream()
				.filter(
					p -> p
						.getQualifier()
						.getClassid()
						.equalsIgnoreCase("orcid"))
				.map(o -> o.getValue())
				.collect(Collectors.toList());
			if (oList.size() > 0)
				author += ", " + oList.get(0);
		}
		return author;

	}

	private static String getListOfValues(Set<String> cfSet) {
		StringWriter sw = new StringWriter();
		cfSet.forEach(value -> sw.append(value + "; "));

		return sw.toString().length() > 0 ? sw.toString().substring(0, sw.toString().length() - 2) : "";
	}

	private static void getDescription(CatalogueEntry out, List<KeyValue> externals, List<Field<String>> value) {
		Iterator<Field<String>> it = value.iterator();
		if (it.hasNext()) {
			out.setNotes(textReplacement(it.next().getValue()));
		} else {
			out.setNotes("");
		}
		it.forEachRemaining(v -> externals.add(KeyValue.newInstance("Description", v.getValue())));
	}

}
