
package eu.dnetlib.dhp.oa.graph.dump.gcat;

import static eu.dnetlib.dhp.common.SparkSessionSupport.runWithSparkSession;

import java.io.Serializable;
import java.util.*;
import java.util.stream.Collectors;

import org.apache.commons.io.IOUtils;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.function.MapFunction;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Encoders;
import org.apache.spark.sql.SaveMode;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.util.LongAccumulator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import eu.dnetlib.dhp.application.ArgumentApplicationParser;
import eu.dnetlib.dhp.oa.graph.dump.Utils;
import eu.dnetlib.dhp.schema.dump.gcat.CatalogueEntry;
import eu.dnetlib.dhp.schema.oaf.Context;
import eu.dnetlib.dhp.schema.oaf.Result;

public class SparkDumpRISISCatalogue implements Serializable {

	private static final Logger log = LoggerFactory.getLogger(SparkDumpRISISCatalogue.class);

	public static void main(String[] args) throws Exception {
		String jsonConfiguration = IOUtils
			.toString(
				SparkDumpRISISCatalogue.class
					.getResourceAsStream(
						"/eu/dnetlib/dhp/oa/graph/gcat/dump_parameters.json"));

		final ArgumentApplicationParser parser = new ArgumentApplicationParser(jsonConfiguration);
		parser.parseArgument(args);

		Boolean isSparkSessionManaged = Optional
			.ofNullable(parser.get("isSparkSessionManaged"))
			.map(Boolean::valueOf)
			.orElse(Boolean.TRUE);
		log.info("isSparkSessionManaged: {}", isSparkSessionManaged);

		final String inputPath = parser.get("sourcePath");
		log.info("inputPath: {}", inputPath);

		final String outputPath = parser.get("outputPath");
		log.info("outputPath: {}", outputPath);

		final String resultClassName = parser.get("resultTableName");
		log.info("resultTableName: {}", resultClassName);

		final String communityName = parser.get("communityName");
		log.info("communityName: {}", communityName);

		Class<? extends Result> inputClazz = (Class<? extends Result>) Class.forName(resultClassName);
		SparkConf conf = new SparkConf();

		runWithSparkSession(
			conf,
			isSparkSessionManaged,
			spark -> {
				Utils.removeOutputDir(spark, outputPath);
				Map<String, LongAccumulator> map = new HashMap<>();
				map.put("dumped_records", spark.sparkContext().longAccumulator("dumped_records"));
				map.put("send_to_dump_records", spark.sparkContext().longAccumulator("send_to_dump_records"));
				map.put("skipped_records", spark.sparkContext().longAccumulator("skipped_records"));
				map.put("void_records", spark.sparkContext().longAccumulator("void_records"));
				execDump(
					spark, inputPath, outputPath, inputClazz, communityName, map);// ,
				// dumpClazz);
				log.info("records send to dump: {}", map.get("send_to_dump_records").value());
				log.info("skipped records : {}", map.get("skipped_records").value());
				log.info("dumped_records : {}", map.get("dumped_records").value());
			});

	}

	public static <I extends Result, O extends eu.dnetlib.dhp.schema.dump.oaf.Result> void execDump(SparkSession spark,
		String inputPath,
		String outputPath,
		Class<I> inputClazz,
		String communityName,
		Map<String, LongAccumulator> map) {// Class<O> dumpClazz) {

		// Set<String> communities = communityMap.keySet();
		Utils
			.readPath(spark, inputPath, inputClazz)
			.map(
				(MapFunction<I, CatalogueEntry>) value -> execMap(
					value, communityName, map),
				Encoders.bean(CatalogueEntry.class))
			.filter(Objects::nonNull)
			.coalesce(1)
			.write()
			.mode(SaveMode.Overwrite)
			.option("compression", "gzip")
			.json(outputPath);

	}

	private static <I extends Result> CatalogueEntry execMap(I value, String community,
		Map<String, LongAccumulator> map) {

		if (value.getDataInfo().getDeletedbyinference() || value.getDataInfo().getInvisible()) {
			// map.get("skipped_records").add(1);
			return null;
		}
		Optional<List<Context>> inputContext = Optional.ofNullable(value.getContext());
		if (!inputContext.isPresent()) {
			map.get("skipped_records").add(1);
			return null;
		}
		if (inputContext.get().stream().map(c -> {
			String id = c.getId();
			if (id.contains("::")) {
				return id.substring(0, id.indexOf("::"));
			}
			return id;
		}).collect(Collectors.toList()).contains(community)) {
			map.get("send_to_dump_records").add(1);
			return Mapper.map(value, map);
		}
		map.get("skipped_records").add(1);
		return null;
//			List<String> toDumpFor = inputContext.get().stream().map(c -> {
//				String id = c.getId();
//				if (id.contains("::")) {
//					id = id.substring(0, id.indexOf("::"));
//				}
//				if (community.equals(id)) {
//					dumpedRecords.add(1);
//					return id;
//				}
//				return null;
//			}).filter(Objects::nonNull).collect(Collectors.toList());
//			if (toDumpFor.size() == 0) {
//				skippedRecords.add(1);
//				return null;
//			}
//			return Mapper.map(value);

	}

}
