
package eu.dnetlib.dhp.oa.graph.gcat;

import java.io.*;
import java.net.URISyntaxException;
import java.util.List;
import java.util.zip.GZIPInputStream;

import org.apache.commons.io.IOUtils;
import org.apache.http.HttpStatus;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;

import com.google.common.collect.Lists;

import eu.dnetlib.dhp.oa.graph.dump.gcat.GCatAPIClient;

/**
 * NEVER EVER ENABLE THIS CLASS UNLESS YOU ABSOLUTELY KNOW WHAT YOU ARE DOING: with the proper parameters set it can
 * drop a D4Science Catalogue
 */
//@Disabled
public class GCatAPIClientTest {

	private static GCatAPIClient client;

	@BeforeAll
	public static void setup() {
		client = new GCatAPIClient();
		client.setApplicationToken("816486a3-60a9-4ecc-a7e0-a96740a90207-843339462");
		client.setGcatBaseURL("https://gcat.d4science.org/gcat/");
	}

	@Test
	public void testList() throws IOException, URISyntaxException {
		System.out.println(client.list(0, 10));
	}

	@Test
	public void testPublishAndPurge() throws IOException, URISyntaxException {
		// The 'name' must be between 2 and 100 characters long and contain only lowercase alphanumeric characters, '-'
		// and '_'.
		// You can validate your name using the regular expression : ^[a-z0-9_\\-]{2,100}$
		String objidentifier = "fake";
		String json = IOUtils
			.toString(getClass().getResourceAsStream("/eu/dnetlib/dhp/oa/graph/dump/gcat/gcat_dat.json"));
		System.out.println("Creating item...");
		Assertions.assertEquals(HttpStatus.SC_CREATED, client.publish(json));
		System.out.println("item created, now listing...");
		Assertions.assertEquals(1, client.list(0, 10).size());
		// and then drop it
		Assertions.assertTrue(client.purge(objidentifier));
		System.out.println("item purged");
	}

	@Test
	public void testPublish() throws IOException, URISyntaxException {
		// The 'name' must be between 2 and 100 characters long and contain only lowercase alphanumeric characters, '-'
		// and '_'.
		// You can validate your name using the regular expression : ^[a-z0-9_\\-]{2,100}$
		String json = IOUtils
			.toString(
				getClass().getResourceAsStream("/eu/dnetlib/dhp/oa/graph/dump/gcat/gcat_software_20201130.json"));
		System.out.println("Creating item...");
		Assertions.assertEquals(HttpStatus.SC_CREATED, client.publish(json));
		System.out.println("item created, now listing...");
		// Assertions.assertEquals(1, client.list(0, 10).size());

	}

	@Test
	public void bulkPublishORP() throws IOException, URISyntaxException {
		BufferedReader reader = new BufferedReader(new FileReader(getClass()
			.getResource("/eu/dnetlib/dhp/oa/graph/dump/gcat/gcat_other.json")
			.getPath()));

		String line;
		while ((line = reader.readLine()) != null) {
			Assertions.assertEquals(HttpStatus.SC_CREATED, client.publish(line));
		}
	}

	@Test
	public void bulkPublishDATS() throws IOException, URISyntaxException {

		BufferedReader reader = new BufferedReader(new FileReader(getClass()
			.getResource("/eu/dnetlib/dhp/oa/graph/dump/gcat/dats_20201126")
			.getPath()));

		String line;
		int count = 1;
		while ((line = reader.readLine()) != null) {
			Assertions.assertEquals(HttpStatus.SC_CREATED, client.publish(line));
			System.out.println(count);
			count++;
		}
	}

	@Test
	public void bulkPublishCompressedSW() throws IOException, URISyntaxException {

		BufferedReader reader = new BufferedReader(
			new InputStreamReader(new GZIPInputStream(new FileInputStream(getClass()
				.getResource("/eu/dnetlib/dhp/oa/graph/dump/gcat/software_20201130.gz")
				.getPath()))));

		String line;
		int count = 1;
		while ((line = reader.readLine()) != null) {
			Assertions.assertEquals(HttpStatus.SC_CREATED, client.publish(line));
			System.out.println(count);
			count++;
		}
	}

	@Test
	public void bulkPublishPUBS() throws IOException, URISyntaxException {
		BufferedReader reader = new BufferedReader(new FileReader(getClass()
			.getResource("/eu/dnetlib/dhp/oa/graph/dump/gcat/gcat_publications")
			.getPath()));

		String line;
		while ((line = reader.readLine()) != null) {
			Assertions.assertEquals(HttpStatus.SC_CREATED, client.publish(line));
		}
	}

	@Test
	public void purgeItem() throws IOException, URISyntaxException {
		String objidentifier = "dedup_wf_001--10160b3eafcedeb0a384fc400fe1c3fa";
		Assertions.assertTrue(client.purge(objidentifier));
		System.out.println("item purged");

	}

	@Test
	public void testPurgeUnexisting() throws IOException, URISyntaxException {
		String id = "1234";
		Assertions.assertFalse(client.purge(id));
	}

	@Test
	public void testPurgeAllEmptyCat() throws IOException, URISyntaxException {
		Assertions.assertEquals(179, client.purgeAll());
	}

	@Test
	public void testPublishAndPurgeAll() throws IOException, URISyntaxException {
		String json = IOUtils
			.toString(getClass().getResourceAsStream("/eu/dnetlib/dhp/oa/graph/dump/gcat/gcat_pub.json"));
		Assertions.assertEquals(HttpStatus.SC_CREATED, client.publish(json));
		System.out.println("item created, now listing...");
		Assertions.assertEquals(1, client.list(0, 10).size());
		// and then drop all
		Assertions.assertEquals(1, client.purgeAll());
	}

	@Test
	public void purgeList() throws IOException, URISyntaxException {
		List<String> toPurge = Lists.newArrayList();
		toPurge.add("dedup_wf_001--f20bb2f571f1fdcb9a66bec850a8267e");
		toPurge.add("od_______166--c5caa1b39d9c7998f0f7c37f948ea097");
		toPurge.add("od______2659--d956cae2b4a87eeaae4291530dfc88cf");
		toPurge.add("od______2659--50fdc84d38782630c8fe73ca66e4e1e9");
		toPurge.add("od______2659--1bbce6c3a47aa79cfc2c2973842d0c2c");
		toPurge.add("od______2659--037571fc3efb68d43ae8a4d8078ddd82");
		toPurge.add("od______2659--7bd3645a599d5c9fd5eb0a5c87b79948");
		toPurge.add("od______3379--50842bb0a03d644a6ed831c0a59d25f4");
		toPurge.add("od_______177--50842bb0a03d644a6ed831c0a59d25f4");
		toPurge.add("od______4325--d6ad4e4111afd06f69c1597a60f09cef");
		toPurge.add("od______3379--d6ad4e4111afd06f69c1597a60f09cef");
		toPurge.add("od_______177--d6ad4e4111afd06f69c1597a60f09cef");
		toPurge.add("od______4325--50842bb0a03d644a6ed831c0a59d25f4");
		toPurge.add("od______4325--b823dc448d06160da67ebdcd1a67c544");
		toPurge.add("od_______177--b823dc448d06160da67ebdcd1a67c544");
		toPurge.add("od______1106--7407d45261b901f936319762b30a66f0");
		toPurge.add("od______3379--b823dc448d06160da67ebdcd1a67c544");
		toPurge.add("od______4325--655f1b9517a0dd16efd05b572f66927b");
		toPurge.add("od______3379--655f1b9517a0dd16efd05b572f66927b");
		toPurge.add("od_______177--655f1b9517a0dd16efd05b572f66927b");
		toPurge.add("od______3379--308718c4498f1c857d9dec8fc8412bed");
		toPurge.add("od_______177--308718c4498f1c857d9dec8fc8412bed");
		toPurge.add("od______4325--308718c4498f1c857d9dec8fc8412bed");
		toPurge.add("od______4325--26025853fbcb01858d58e3c268d144ce");
		toPurge.add("od_______177--26025853fbcb01858d58e3c268d144ce");
		toPurge.add("od______3379--4af67cb057f92a8c276c3aae56980430");
		toPurge.add("od_______177--4af67cb057f92a8c276c3aae56980430");
		toPurge.add("od______4325--4af67cb057f92a8c276c3aae56980430");
		toPurge.add("od______3379--1c384c7771d6d5ec0b2b14264d0af8cd");
		toPurge.add("od_______177--1c384c7771d6d5ec0b2b14264d0af8cd");
		toPurge.add("od______4325--1c384c7771d6d5ec0b2b14264d0af8cd");
		toPurge.add("od_______177--15a65e2433929cc77ae7b10fd56c1e9e");
		toPurge.add("od______4325--15a65e2433929cc77ae7b10fd56c1e9e");
		toPurge.add("od______3379--15a65e2433929cc77ae7b10fd56c1e9e");
		toPurge.add("od_______177--c5caa1b39d9c7998f0f7c37f948ea097");
		toPurge.add("od______3379--c5caa1b39d9c7998f0f7c37f948ea097");
		toPurge.add("od_______177--11df09fd7a9ad36f0de546ea991182ce");
		toPurge.add("od______3379--11df09fd7a9ad36f0de546ea991182ce");
		toPurge.add("od______4325--11df09fd7a9ad36f0de546ea991182ce");
		toPurge.add("dedup_wf_001::2fb2a1e02bfae184789181009d59232c");
		toPurge.add("od_______177--39688c4bc9fd811e55f914e58701536d");
		toPurge.add("od______3379--39688c4bc9fd811e55f914e58701536d");
		toPurge.add("od______4325--39688c4bc9fd811e55f914e58701536d");
		toPurge.add("od______4325--200c9c70dff2e86dad6ba555381027fc");
		toPurge.add("od______3379--200c9c70dff2e86dad6ba555381027fc");
		toPurge.add("od_______177--200c9c70dff2e86dad6ba555381027fc");
		toPurge.add("od______3379--cf54f6149e7427d77dd37fccc3a0c747");
		toPurge.add("od______3379--1f50518c04e2c0966425a350def1f82a");
		toPurge.add("od______4325--1f50518c04e2c0966425a350def1f82a");
		toPurge.add("od_______177--1f50518c04e2c0966425a350def1f82a");
		toPurge.add("od______3379--bff6c5b35457f7f51d52d41323226663");
		toPurge.add("od_______177--bff6c5b35457f7f51d52d41323226663");
		toPurge.add("od______4325--bff6c5b35457f7f51d52d41323226663");
		toPurge.add("od______2659--cbdcbe3865868c15680d95c7f83c3ff7");
		toPurge.add("dedup_wf_001--569da4f719f51eb07f23548000e57d72");
		toPurge.add("od_______177--bf88e4876d9c5a9720ca3fefe3ce93ea");
		toPurge.add("od______3379--bf88e4876d9c5a9720ca3fefe3ce93ea");
		toPurge.add("od______4325--bf88e4876d9c5a9720ca3fefe3ce93ea");
		for (String name : toPurge) {
			System.out.println("Purging " + name);
			if (client.purge(name))
				System.out.println("Purged");
			else {
				System.out.println("Failed");
			}
		}

	}
}
